#!/usr/bin/env python
#
import argparse
import os
import re
import sys
from modules.ProcUtils import httpdl
from urllib.parse import urlparse

def retrieveURL(request,localpath='.', uncompress=False, verbose=0,force_download=False):
        if args.verbose:
            print("Retrieving %s" % request.rstrip())

        server = "oceandata.sci.gsfc.nasa.gov"
        parsedRequest = urlparse(request)
        netpath = parsedRequest.path
        if parsedRequest.netloc:
            server = parsedRequest.netloc
        else:
            if not re.match("getfile",netpath):
                netpath = '/ob/getfile/' + netpath

        if parsedRequest.query:
            netpath = netpath + '?' + parsedRequest.query

        return httpdl(server, netpath, localpath=localpath, uncompress=uncompress, verbose=verbose,force_download=force_download)


if __name__ == "__main__":
    # parse command line
    parser = argparse.ArgumentParser(
        description='Download files archived at the OB.DAAC',
        epilog='''
Provide one of either filename, --filelist or --http_manifest.
NOTE: For authentication, a valid .netrc file in the user home ($HOME) is required, e.g.:
    machine urs.earthdata.nasa.gov login USERNAME password PASSWD
'''
    )
    parser.add_argument('-v', '--verbose', help='print status messages',
                        action='count',default=0)
    parser.add_argument('filename', nargs='?', help='name of the file (or the URL of the file) to retreive')
    parser.add_argument('--filelist',
                        help='file containing list of filenames to retreive, one per line')    
    parser.add_argument('--http_manifest',
                        help='URL to http_manifest file for OB.DAAC data order')
    parser.add_argument('--odir',
                        help='full path to desired output directory; defaults to current working directory: %s' % os.getcwd(),
                        default=os.getcwd())
    parser.add_argument('--uncompress',action="store_true",
                        help='uncompress the retrieved files (if compressed)',
                        default=False)
    parser.add_argument('--force',action='store_true',
                        help='force download even if file already exists locally',
                        default=False)
    args = parser.parse_args()

    filelist = []

    if args.http_manifest:
        status = retrieveURL(args.http_manifest,verbose=args.verbose,force_download=True)
        if status:
            print("Whoops...problem retrieving %s (received status %d)" % (args.http_manifest,status))
            sys.exit("Bailing out...")
        else:
            with open('http_manifest.txt') as flist:
                for filename in flist:
                    filelist.append(filename.rstrip())
    elif args.filename:
        filelist.append(args.filename)
    elif args.filelist:
        with open(args.filelist) as flist:
            for filename in flist:
                filelist.append(filename.rstrip())

    if not len(filelist):
        parser.print_usage()
        sys.exit("Please provide a filename (or list file) to retrieve")

    outpath = os.path.abspath(os.path.expanduser(os.path.expandvars(args.odir)))
    if args.verbose:
        print("Output directory: %s" % outpath)

    for request in filelist:
        status = retrieveURL(request,localpath=outpath, uncompress=args.uncompress, verbose=args.verbose,force_download=args.force)
        if status:
            if status == 304: 
                if args.verbose:
                    print("%s is not newer than local copy, skipping download" % request)
            else:
                print("Whoops...problem retrieving %s (received status %d)" % (request,status))